@@ -23,11 +23,12 @@ if str(version_info.major) == "2":
 
 class Plugin(object):
     def __init__(self, config):
-        self._name = 'saml'
-        self._description = 'Selection a Role from SAML.'
-        self._cloudtoken_dir = "{0}/.config/cloudtoken".format(os.path.expanduser('~'))  # TODO: Pull from config.yaml
+        self._name = "saml"
+        self._description = "Selection a Role from SAML."
+        self._cloudtoken_dir = "{0}/.config/cloudtoken".format(os.path.expanduser("~"))  # TODO: Pull from config.yaml
         self._cloudtoken_json_filename = "{0}/tokens.json".format(self._cloudtoken_dir)  # TODO: Pull from config.yaml
         self._config = config
+        self._defaults = {}
 
     def __str__(self):
         return __file__
@@ -39,33 +40,35 @@ class Plugin(object):
     @staticmethod
     def arguments(defaults):
         parser = argparse.ArgumentParser()
-        parser.add_argument("-l",
-                            "--list",
-                            dest="list",
-                            action="store_true",
-                            help="List all roles available to the user then exit.")
-        parser.add_argument("-f",
-                            "--filter",
-                            dest="filter",
-                            default=defaults.get('filter', None),
-                            help="Only show roles matching filter.")
-        parser.add_argument("-n",
-                            "--numbers-only",
-                            dest="numbers_only",
-                            action="store_true",
-                            help="Only list role numbers. Useful for automation.")
-        parser.add_argument("--refresh",
-                            action="store_true",
-                            help="Refresh tokens for last selected Role.")
-        parser.add_argument("-r",
-                            "--role-number",
-                            default=defaults.get('role-number', None),
-                            help="Specify a role without prompting you to select from the list.")
+        parser.add_argument(
+            "-l", "--list", dest="list", action="store_true", help="List all roles available to the user then exit."
+        )
+        parser.add_argument(
+            "-f",
+            "--filter",
+            dest="filter",
+            default=defaults.get("filter", None),
+            help="Only show roles matching filter.",
+        )
+        parser.add_argument(
+            "-n",
+            "--numbers-only",
+            dest="numbers_only",
+            action="store_true",
+            help="Only list role numbers. Useful for automation.",
+        )
+        parser.add_argument("--refresh", action="store_true", help="Refresh tokens for last selected Role.")
+        parser.add_argument(
+            "-r",
+            "--role-number",
+            default=defaults.get("role-number", None),
+            help="Specify a role without prompting you to select from the list.",
+        )
         return parser
 
     @staticmethod
     def filter_roles(pattern, roles):
-        p = re.compile(r'{0}'.format(pattern), re.IGNORECASE)
+        p = re.compile(r"{0}".format(pattern), re.IGNORECASE)
         filtered_list = list()
 
         for b in range(1, len(roles) + 1):
@@ -77,19 +80,24 @@ class Plugin(object):
 
     def create_role_list(self, args, saml_response):
         saml_response = b64decode(saml_response)
-        namespaces = {'ds': 'http://www.w3.org/2000/09/xmldsig#',
-                      'html': 'http://www.w3.org/1999/xhtml',
-                      'saml': 'urn:oasis:names:tc:SAML:2.0:assertion',
-                      'samlp': 'urn:oasis:names:tc:SAML:2.0:protocol',
-                      'xs': 'http://www.w3.org/2001/XMLSchema',
-                      'xsi': 'http://www.w3.org/2001/XMLSchema-instance'}
+        namespaces = {
+            "ds": "http://www.w3.org/2000/09/xmldsig#",
+            "html": "http://www.w3.org/1999/xhtml",
+            "saml": "urn:oasis:names:tc:SAML:2.0:assertion",
+            "samlp": "urn:oasis:names:tc:SAML:2.0:protocol",
+            "xs": "http://www.w3.org/2001/XMLSchema",
+            "xsi": "http://www.w3.org/2001/XMLSchema-instance",
+        }
         saml = ET.fromstring(saml_response)
-        a = saml.find(".//{urn:oasis:names:tc:SAML:2.0:assertion}Attribute[@Name='https://aws.amazon.com/SAML/Attributes/Role']", namespaces=namespaces)  # noqa
+        a = saml.find(
+            ".//{urn:oasis:names:tc:SAML:2.0:assertion}Attribute[@Name='https://aws.amazon.com/SAML/Attributes/Role']",
+            namespaces=namespaces,
+        )  # noqa
 
         role_list = []
-        for b in a.getchildren():
-            arns = b.text.split(',')
-            if ':saml-provider/' in arns[0]:
+        for b in a:
+            arns = b.text.split(",")
+            if ":saml-provider/" in arns[0]:
                 role_list.append([arns[1], arns[0]])
             else:
                 role_list.append([arns[0], arns[1]])
@@ -139,66 +147,86 @@ class Plugin(object):
 
     @staticmethod
     def assume_role_with_saml(saml_response, role):
-        client = boto3.client('sts')
+        client = boto3.client("sts")
 
         role_arn = role[0]
         principle_arn = role[1]
 
-        temp_credentials = client.assume_role_with_saml(RoleArn=role_arn,
-                                                        PrincipalArn=principle_arn,
-                                                        SAMLAssertion=saml_response)
+        temp_credentials = client.assume_role_with_saml(
+            RoleArn=role_arn, PrincipalArn=principle_arn, SAMLAssertion=saml_response
+        )
 
         logging.debug("AssumeRoleWithSaml Response: {0}".format(temp_credentials))
 
         credentials = {
-            'AccessKeyId': temp_credentials['Credentials']['AccessKeyId'],
-            'AccountId': role_arn.split(':')[4],
-            'Code': 'Success',
-            'Expiration': temp_credentials['Credentials']['Expiration'].strftime('%Y-%m-%dT%H:%M:%SZ'),
-            'Issuer': temp_credentials['Issuer'],
-            'LastRole': role_arn,
-            'LastUpdated': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
-            'SAMLProvider': principle_arn,
-            'SecretAccessKey': temp_credentials['Credentials']['SecretAccessKey'],
-            'Token': temp_credentials['Credentials']['SessionToken'],
-            'Type': 'AWS-HMAC',
+            "AccessKeyId": temp_credentials["Credentials"]["AccessKeyId"],
+            "AccountId": role_arn.split(":")[4],
+            "Code": "Success",
+            "Expiration": temp_credentials["Credentials"]["Expiration"].strftime("%Y-%m-%dT%H:%M:%SZ"),
+            "Issuer": temp_credentials["Issuer"],
+            "LastRole": role_arn,
+            "LastUpdated": datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ"),
+            "SAMLProvider": principle_arn,
+            "SecretAccessKey": temp_credentials["Credentials"]["SecretAccessKey"],
+            "Token": temp_credentials["Credentials"]["SessionToken"],
+            "Type": "AWS-HMAC",
         }
 
         return credentials
 
+    def _load_defaults(self):
+        """Load defaults.
+        """
+        if self._config.get("defaults", None) and self._config["defaults"].get(self._name, None):
+            for key, value in self._config["defaults"][self._name].items():
+                self._defaults[key] = value
+
     def execute(self, data, args, flags):
-        if flags.get('daemon_mode', False):
+        self._load_defaults()  # Load defaults if they exist.
+
+        if flags.get("daemon_mode", False):
             for d in data:
-                if dict(d)['plugin'] == self._name:
-                    last_role = dict(d)['data']['LastRole']
+                if dict(d)["plugin"] == self._name:
+                    last_role = dict(d)["data"]["LastRole"]
                     break
             args.filter = last_role
 
-        # We expect the SAML response to be passed from the previously run plugin.
-        try:
-            saml_response = dict(data[-1])['data']
-        except Exception:
-            print("Unable to load SAML response from previous plugin. Exiting.")
-            exit(1)
+        logging.debug("Checking if any plugin has nominated that it has a SAMLResponse.")
+        saml_response = None
+        for payload in reversed(data):
+            if payload.get("contains_samlresponse"):
+                logging.debug("Plugin {} says it has a SAMLResponse.".format(payload["plugin"]))
+                try:
+                    saml_response = payload["data"]
+                    break
+                except KeyError:
+                    raise SystemExit("Unable to load SAML response from {} plugin. Exiting.".format(payload["plugin"]))
+
+        if not saml_response:
+            logging.debug("No plugin has specified it has a SAMLResponse. Will default to previous plugin.")
+            try:
+                saml_response = dict(data[-1])["data"]
+            except Exception:
+                raise SystemExit("Unable to load SAML response from previous plugin. Exiting.")
+
         logging.debug("SAMLResponse: {0}".format(b64decode(saml_response)))
 
         roles = self.create_role_list(args, saml_response)
         if not roles:
-            print("No roles found in SAML Response.")
-            exit(1)
+            raise SystemExit("No roles found in SAML Response.")
         logging.debug("Roles: {0}".format(roles))
 
         if args.numbers_only:
             self.list_roles(roles, numbers_only=True)
-            exit(0)
+            raise SystemExit(0)
 
         if args.refresh:
             with open(self._cloudtoken_json_filename) as fh:
                 token_json = json.loads(fh.read())
-            selected_role = [token_json['LastRole'], token_json['SAMLProvider']]
+            selected_role = [token_json["LastRole"], token_json["SAMLProvider"]]
         elif len(roles) == 1:
             # Don't want this to print every time in daemon mode.
-            if flags.get('daemon_mode', False):
+            if flags.get("daemon_mode", False):
                 if not args.quiet:
                     print("Using role {0}".format(last_role))
             else:
@@ -215,5 +243,12 @@ class Plugin(object):
 
         credentials = self.assume_role_with_saml(saml_response, selected_role)
 
-        data.append({'plugin': self._name, 'data': credentials})
+        updated = False
+        for payload in data:
+            if payload["plugin"] == self._name:
+                payload["data"] = credentials
+                updated = True
+        if not updated:
+            data.append({"plugin": self._name, "data": credentials})
+
         return data
